# -*- perl -*-

# t/001_load.t - check module loading and create testing directory

use Test::More tests => 2;

use lib qw( t/. );

BEGIN { use_ok( 'PDF::Template' ); }

my $object = PDF::Template->new ();
isa_ok ($object, 'PDF::Template');
